#ifndef _MK2VSC_H_
#define _MK2VSC_H_

#include <velib/velib_inc_mk2.h>
#include <velib/mk2/vebus_device.h>

#include "vscFile.h"

#define MK2RVSC_VERSION					"1.06"

#define MAX_PATH_LEN					256
#define VEBUS_MAX_SETTINGS_COUNT		254					/* This includes the grid compliance settings */
#define VEBUS_GRID_COMPLIANCE_SETTINGS_START_INDEX		128 /* If setting ID 128 is not supported,
															 * there is no need to continue reading. */

#define N_COMM_FAIL_RESTART_MK2         1      /* number of Mk2 restart attempts when things go wrong */
#define N_MULTI_RESET_CNT				1	   /* number of Multi reset attempts when vebus config is not sane */
#define N_VEBUSFR_RETRANSMITS			2      /* maximum number of vebus frame retransmits */

#define WD_TIMEOUT_TIME_50MS			6000   /* 5 minutes timeout in case something goes totaly wrong */
/* Writing VLL code takes 1 second per block. With a maximum of 255 blocks a write takes 255 seconds */

typedef enum {
	READ_SETTINGS,
	WRITE_SETTINGS,
	LIST_PRODUCTS
} Mk2VscMode;

typedef void Mk2VscCallback(int retCode);
typedef void DelayedExecCallBack(void);

typedef struct Mk2VscStateS {
	Mk2VscMode mode;
	VscSettingInfoSection* pVscSettingInfoSectionIn;  /* Info settings read from the product */
	VscSettingInfoSection* pVscSettingInfoSectionOut; /* Info settings to be written to the product */
	VscSettingDataSection* pVscSettingDataSectionIn;  /* Data settings read from the product */
	VscSettingDataSection* pVscSettingDataSectionOut; /* Data settings to be written to the product */
	veBool multiResetRequired;	/* After updating VLL code and/or the VEBus settings the Multi should be reset */
	veBool multiVebusConfigCheck; /* True while checking if vebus config is sane */
	struct VebusDeviceS* vebusDevice;
	struct VebusDeviceS* vebusDevices[CFG_VEBUS_DEVICES];
	int nVebusDevices;
	Mk2VscCallback* taskDoneCb;
	char fileName[MAX_PATH_LEN];
	VscFile vscFile;
	un8 vebusSettingsCount;		/* Number of settings we need to read */
	un8 iSetting;				/* setting index */
	un8 iSettingLast;			/* last supported setting */
	un8 iDev;					/* vebusDevice index  */
	un8 restartCnt;				/* number of restart attempts after a comm failure */
	un8 multiResetCnt;			/* number of Multi reset attempts when vebus config is not sane */
	un8 retransmitCnt;			/* number of retransmits. Some vebus frames have no reply */
	un32 usrInputWaitTimeMs;	/* used to be able to timeout on DBUS user input */
} Mk2VscState;


/* DBUS status definitions */
#define MK2VSC_ST_INIT							10
#define MK2VSC_ST_QUERY_DEVICE					11   /* this also reads the setting info */
#define MK2VSC_ST_DONE							12
#define MK2VSC_ST_READ_SETTING_DATA				21
#define MK2VSC_ST_READ_VLL_CODE					22
#define MK2VSC_ST_READ_VEBUS_SETTINGS			23
#define MK2VSC_ST_WRITE_SETTINGS_INFO			30
#define MK2VSC_ST_WRITE_SETTINGS_DATA			31
#define MK2VSC_ST_WRITE_VLL_CODE				32
#define MK2VSC_ST_WRITE_VEBUS_SETTINGS			33
#define MK2VSC_ST_RESETTING_VEBUS_PRODUCTS		40

/* Error/application exit code definitions */
#define MK2VSC_ERR_MK2_COMM						30 /*!< MK2 communication error */
#define MK2VSC_ERR_PROD_ADDR					31 /*!< Product address not reachable */
#define MK2VSC_ERR_FIRMWARE_VERSION				32 /*!< Incompatible MK2 firmware version */
#define MK2VSC_ERR_NO_VEBUS_PRODUCT				33 /*!< No VEBus product was found */
#define MK2VSC_ERR_VEBUS_ADDR_OVERFLOW			34 /*!< Too many devices on the VEBus */
#define MK2VSC_ERR_TIMEOUT						35 /*!< Timed out */
#define MK2VSC_ERR_PASSWORD						36 /*!< Wrong password */

#define MK2VSC_ERR_MALLOC						40 /*!< Malloc error */

#define MK2VSC_ERR_FILE_OPEN					50 /*!< File open error */
#define MK2VSC_ERR_FILE_WRITE					51 /*!< File write error */
#define MK2VSC_ERR_FILE_READ					52 /*!< File read error */
#define MK2VSC_ERR_FILE_CHECKSUM				53 /*!< File checksum error */
#define MK2VSC_ERR_FILE_VERSION					54 /*!< File incompatible version number */
#define MK2VSC_ERR_FILE_SECTION					55 /*!< File section not found */
#define MK2VSC_ERR_FILE_FORMAT					56 /*!< File format error */
#define MK2VSC_ERR_FILE_PROD_ID					57 /*!< Product target number does not match file */
#define MK2VSC_ERR_FILE_EXPIRED					58 /*!< File is too old >*/

#define MK2VSC_ERR_VEBUS_SET_WRITE				60 /*!< VEBus setting write failed */
#define MK2VSC_ERR_VEBUS_SET_INC				61 /*!< Incompatible VEBus settings */

#define MK2VSC_ERR_VLL_WRITE					70 /*!< VLL code write failed */
#define MK2VSC_ERR_VLL_READ						71 /*!< VLL code read failed */

void mk2vscDelayedExec(DelayedExecCallBack* cb, un8 n50msCount);
void mk2vscSetDefaults(void);
void mk2vscSetMode(Mk2VscMode mode);
Mk2VscMode mk2vscGetMode(void);
void mk2vscSetIgnoreIdFlag(veBool flag);
veBool mk2vscGetIgnoreIdFlag(void);
void mk2vscSetFileName(char* name);
void mk2vscEnableDBusAcknowledge(void);

#endif							/* _MK2VSC_H_ */
