#ifndef _VEBUS_VLL_H_
#define _VEBUS_VLL_H_

#include <velib/base/types.h>
#include <velib/mk2/vebus_device.h>
#include <velib/velib_inc_mk2.h>

#include "sa_search.h"

#define MAX_VLL_BLOCK_NR	0xFF /* The maximum VLL block number index (used for probing available blocks) */
#define VLL_BLOCK_SIZE		64

typedef enum {
	BUSY,
	SUCCESS,
	FAILED,
	FF_BLOCK,					/* success, the 64 bytes block is filled with 0xFF */
	CMD_NOT_SUPPORTED,
	INVALID_BLOCK
} VllReadResult;

typedef void VebusVllProbeCallback(VllReadResult result);

typedef struct {
	un8 blockNr;
	un8 subBlockCmd;
	char* pData;
	struct VebusDeviceS* dev;
	Mk2Callback* writeDoneCallback;
} VllWriteState;

typedef struct {
	VllReadResult result;
	char data[VLL_BLOCK_SIZE];
} VllReadBuffer;

typedef struct {
	un8 blockNr;
	un8 subBlockNr;
	struct VebusDeviceS* dev;
	Mk2Callback* readDoneCallback;
	VllReadBuffer* pBuf;
} VllReadState;

typedef struct {
	un16* pNBlocks;				/* Total number of blocks found */
	SaSearchState sasst;
	VllReadResult result;
	struct VebusDeviceS* dev;
	VebusVllProbeCallback* probeDoneCallback;
} VllProbeState;


veBool vebusVllWriteBlock(struct VebusDeviceS* dev, Mk2Callback* cb, un8 blockNr, char* pData);
veBool vebusVllWriteBlockAsPrevious(struct VebusDeviceS* dev, Mk2Callback* cb, un8 blockNr);
veBool vebusVllReadBlock(struct VebusDeviceS* dev, Mk2Callback* cb, un8 blockNr, VllReadBuffer* pBuf);
veBool vebusVllProbeNBlocks(struct VebusDeviceS* dev, VebusVllProbeCallback* cb, un16* pNBlocks);

#endif	/* _VEBUS_VLL_H */
