#ifndef _VSCFILE_H_
#define _VSCFILE_H_
#include <stdio.h>

#include <velib/mk2/vebus_system.h>
#include <velib/velib_config_app.h>
#include <velib/mk2/vebus_device.h>
#include <velib/velib_inc_mk2.h>

#include "vebus_vll.h"

#define VSC_SF_MAX_ID_STR_LEN			40 /* max file/section identifier string length (includes 2 length bytes) */

#define VSC_SF_VERSION					1

#define VSC_SF_N_VEBUS_SETTINGS			9
#define VSC_SF_SERIAL_NUMBER_LEN		11
#define VSC_SF_N_PASSWORDS				7
#define VSC_SF_PASSWORD_LEN				8

#define VSC_SF_FILE_ID_STR				"VEConfig setting section file"
#define VSC_SF_SECT_ID_STR_DATA			"BareSettingData"
#define VSC_SF_SECT_ID_STR_INFO			"BareSettingInfo"
#define VSC_SF_SECT_VERSION_DATA		2
#define VSC_SF_SECT_VERSION_INFO		2

#define VSC_SF_PASSWORD_VALID_TIME_SEC	604800 /* Passwords are valid +- 7 days. (incase VE_Configure clock is off) */
#define VSC_SF_SETTING_VALID_TIME_SEC	86400  /* Settings are valid 24 hours after being read. */

#define VEBUS_SETTING_SHORT_ID_INDEX	8 /* the last byte of the VEBus settings contains the short ID */


typedef struct
{
	union {
		un32 uniqueTargetNumber;
		un32 firmwareVersion;
	} key;
	un32 sStartAddr;
	un32 sEndAddr;
} VscFileSectDescr;


typedef struct {
	un16 nSettingInfo;
	un16 nSettingData;
	VscFileSectDescr settingInfo[CFG_VEBUS_DEVICES];
	VscFileSectDescr settingData[CFG_VEBUS_DEVICES];
	VscFileSectDescr systemConfig; /* There can only be one systemConfig section */
} VscFileIndex;


typedef struct {
	FILE* pFile;
	un32 startAddr; /* Start of the vsc file within a vms file. 0 if stand alone */
	un32 endAddr;
	VscFileIndex index;
} VscFile;


typedef struct {
	un16 size;
	char data[MAX_VLL_BLOCK_NR * VLL_BLOCK_SIZE];
} VscVllSectionData;


typedef struct
{
	un16 length;
	un8 password[VSC_SF_PASSWORD_LEN];
} VscAccessPassword;

typedef struct {
	sn16 scale;
	sn16 offset;
	un16 defaultValue;
	un16 minimum;
	un16 maximum;
} VscVebusSettingInfo;

typedef struct {
	un32 uniqueTargetNumber;
	un32 compFirmwareNumber;	/* This indicates if a version number is compatibel */
	un8 identification;
	un32 firmwareVersionNumber;
	un16 versionFrameRemainderLength;
	char versionFrameRemainder[VEBUS_VERSION_REMAINDER_LEN];
	un8 type;
	un8 vebusSetting[VSC_SF_N_VEBUS_SETTINGS];
	un16 serialNumberLength;
	un8 serialNumber[VSC_SF_SERIAL_NUMBER_LEN];
	un8 nPasswords;
	VscAccessPassword passwords[VSC_SF_N_PASSWORDS];
	un64 validTime;             /* Time at which VE Configure wrote the password to the file */
	un64 settingsTime;			/* Time at which settings where written to the file */
	un16 settingCount;			/* in file this corresponds to bsdLength in bytes */
	un16 settingData[CFG_VEBUS_SETTINGS_COUNT];
	VscVllSectionData vllCode;
} VscSettingDataSection;

typedef struct {
	un32 firmwareVersionNumber;
	un8 type;
	un16 settingCount;			/* in file this corresponds to bsiLength in bytes */
	VscVebusSettingInfo settingInfo[CFG_VEBUS_SETTINGS_COUNT];
	un16 flagAccessArrayCount;
	un8 flagAccessArray[CFG_VEBUS_FLAG_REGISTER_COUNT * 16]; /* One flag access byte for each flag bit*/
	un16 settingsAccessArrayCount;
	un8 settingsAccessArray[CFG_VEBUS_SETTINGS_COUNT];
} VscSettingInfoSection;


char* vscFileBufWriteUn16 (un16 val, char* wp);
char* vscFileBufWriteUn32 (un32 val, char* wp);
char* vscFileBufWriteUn64 (un64 val, char* wp);
char* vscFileBufReadUn16 (un16* pVal, char* rp);
char* vscFileBufReadUn32 (un32* pVal, char* rp);
char* vscFileBufReadUn64 (un64* pVal, char* rp);

veBool vscFileGetSettingInfoIndex (VscFile* pVscFile, un32 firmwareVersion, un16* pIndex);
veBool vscFileGetSettingDataIndex (VscFile* pVscFile, un32 targetNumber, un16* pIndex);

int vscFileOpen (VscFile* pVscFile, char* fileName, char mode);
void vscFileClose (VscFile* pVscFile);
int vscFileLoadSettingInfo (VscFile* pVscFile, VscSettingInfoSection* pSetInfo, un32 firmwareVersion);
int vscFileLoadSettingData (VscFile* pVscFile, VscSettingDataSection* pSetData, un32 targetNumber);
int vscFileSaveSettingInfo (VscFile* pVscFile, VscSettingInfoSection* pSetInfo);
int vscFileSaveSettingData (VscFile* pVscFile, VscSettingDataSection* pSetData);

void vscFileInitSettingInfoSection (VscSettingInfoSection* pSetInfo);
void vscFileInitSettingDataSection (VscSettingDataSection* pSetData);
void vscFileSettingInfoSectionToLog (VscSettingInfoSection* pSetInfo);
void vscFileSettingDataSectionToLog (VscSettingDataSection* pSetData);
#endif	/* _VSCFILE_H_ */
