#include <velib/base/base.h>
#include <velib/platform/console.h>

#include <velib/utils/ve_getopt.h>

#include <stdio.h>
#include <stdlib.h>

#include "mk2vsc.h"

#define N_MAX_OPTIONS	10

static int nConsoleOptions = 0;

struct option consoleOptions[] =
{
	{"read",no_argument,0,'r'},
	{"write",no_argument,0,'w'},
	{"file", required_argument, 0,'f'},
	{"list", no_argument, 0, 'l'},
	{0, 0, 0, 0}
};

void consoleUsage(char* program)
{
	printf("%s version %s\n", program, MK2RVSC_VERSION);
	printf("\n");

	printf("A program to realize \"Remote VEConfigure\"\n");
	printf("NOTE: This requires a configured mk2 network.\n");
	printf("      During VLL code update the Multi output will be disabled.\n");

	printf("Usage:\n");
	printf("MK2VSC OPTIONS:\n");
	printf("\n");
	printf("  -r, --read\n");
	printf("   Read the settings from the product and generate a vsc file.\n");
	printf("\n");

	printf("  -w, --write\n");
	printf("   Write the settings to the product.\n");
	printf("\n");

	printf("  -f, --file\n");
	printf("   Custom file name. If the extention is ommited it is added by the application.\n");
	printf("   Extentions:\n");
	printf("     .rvsc for a system with a single Multi (File can be opened directly with \"VEConfigure\")\n");
	printf("     .rvms for a system with multiple Multi (File should be opened with \"VE.Bus System Configurator\")\n");
	printf("   The default file name is the unique target id plus extention.\n");
	printf("\n");

	printf("  -l, --list\n");
	printf("   List connected products.\n");
	printf("\n");

	pltOptionsUsage();
	printf("Victron Energy B.V.\n");
}

veBool consoleOption(int flag)
{
	if (nConsoleOptions == 0) {
		mk2vscSetDefaults();
	}

	switch (flag) {
		case 'r':
			mk2vscSetMode(READ_SETTINGS);
			break;
		case 'w':
			mk2vscSetMode(WRITE_SETTINGS);
			break;
		case 'l':
			mk2vscSetMode(LIST_PRODUCTS);
			break;
		case 'f':
			mk2vscSetFileName (optarg);
			break;
	}

	nConsoleOptions++;

	return veTrue;
}

veBool consoleArgs(int argc, char *argv[])
{
	VE_UNUSED(argv);

	if (nConsoleOptions == 0)
		mk2vscSetDefaults();	/* No console options, use defaults */


	if (argc != 0) {
		printf("error - no arguments are expected - missing a '-' or '--'?\n");
		return veFalse;
	}

	return veTrue;
}
