#include <stdlib.h>

#include <velib/velib_inc_mk2.h>

#include <velib/base/base.h>
#include <velib/base/ve_string.h>
#include <velib/mk2/hal.h>
#include <velib/mk2/vebus_items.h>
#include <velib/platform/console.h>
#include <velib/platform/plt.h>
#include <velib/types/ve_item.h>
#include <velib/types/ve_str.h>
#include <velib/types/variant_print.h>
#include <velib/types/ve_dbus_item.h>
#include <velib/utils/ve_logger.h>

#include "rvsc_dbus_service.h"
#include "mk2vsc.h"

extern VebusDevice* masterDevice;

/** The root of the tree to locate values and format them etc */
static VeItem root;

/** dbus connection */
struct VeDbus *dbus;

/**
 * The vebus system found on the bus containing the values also accessible
 * from the tree.
 */
static VeBusItems veBusItems;

static VeItem *state;
static VeItem *cmd;

static char const version[] = MK2RVSC_VERSION;

char const *pltProgramVersion(void)
{
	return version;
}

void rvscDBusUpdate(void)
{
	veDbusItemUpdate(dbus);
}

/*
 * Create a vebus system object and the corresponding item tree.
 *
 * The tree with items can be used to navigate through the values in
 * vebus system, format / set / get them etc.
 */
VeBusItems* createVebusSystem(VeItem* root)
{
	VeVariant tmp;
	VeItem *item;

	state = veItemGetOrCreateUid(root, "State");
	cmd = veItemGetOrCreateUid(root, "Command");
	veItemSet(cmd, veVariantSn32(&tmp, 0));

	item = veItemGetOrCreateUid(root, "Mgmt/ProcessName");
	veItemSet(item, veVariantStr(&tmp, pltProgramName()));

	item = veItemGetOrCreateUid(root, "Mgmt/ProcessVersion");
	veItemSet(item, veVariantStr(&tmp, version));

	return &veBusItems;
}

void rvscDBusInit(void)
{
	VeStr s;

	createVebusSystem(&root);

	veStrNew(&s, 100, 100);
	veStrAdd(&s, "com.victronenergy.mk2vsc");

	dbus = veDbusGetDefaultBus();

	if (!dbus) {
		veStrFree(&s);
		logE("dbus", "connection error, giving up");
		pltExit(10);
		return;
	}

	veDbusItemInit(dbus, &root);
	veDbusChangeName(dbus, s.data);
	veStrFree(&s);
}

void rvscDBusSetState(int st)
{
	VeVariant tmp;

	veItemSet(state, veVariantSn32(&tmp, st));
}

void rvscDBusResetCmd(void)
{
	VeVariant tmp;

	veItemSet(state, veVariantSn32(&tmp, 0));
}

int rvscDBusGetCmd(void)
{
	VeVariant tmp;

	veItemValue(cmd, veVariantSn32(&tmp, 0));

	return (int) tmp.value.SN32;
}
